'use strict';

const Promise = require('bluebird');
const Joi = require('joi');

const Provider = require('./provider');
const localProviders =  require('./providers');

class MicroKit {
  constructor(options, factory) {
    this.options = options || [];
    this.factory = factory;

    // creates providers
    this._providers = _.mapValues(options.providers, (options, providerName) => {
      const providerType = _.get(options, 'type');

      Joi.assert(providerType, Joi.string().required());

      return this.factory.createProvider(this, providerType, options);
    });

    // creates controllers
    this._controllers = _.mapValues(options.controllers, (options, name) => {
      const providerName = _.get(options, 'use');

      Joi.assert(providerType, Joi.string().required());

      const provider = this.getProvider(providerName);
      return this.factory.createController(provider, options, this);
    });

    // initialize all providers
    this._ready = Promise.map(_.values(this._providers), provider => {
      return provider.initialize()
    });

    // expose controllers
    _.forEach(this._controllers, (controller, name) => this[name] = controller);
  }

  /**
   * Returns promise that resolves when microkit is ready
   * @return {Promise} promise that resolves when microkit is ready
   */
  get ready() {
    return this._ready;
  }

  /**
   * Gets provider by name
   * @param {string} name provider name
   */
  getProvider(name) {
    return _.get(this._providers, name);
  }

  dispose() {
    return Promise.map(_.values(this.providers), provider => {
      return provider.dispose();
    });
  }
}

module.exports = MicroKit;
